import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

//Note that this project has been tested and correctly run under jdk 1.3 and MS Visual j++ 6.0

/**The class containing main method.
 * 
 * Please read the DesignDescription.pdf file for more detail on how to use this program and the functionality it provides
 * 
 *			The demo will always start with the graph of a tetrahedron.
 *			The graph can be changed by selecting the file name in the menu at the bottom right.
 *			Every time the graph file is loaded, the graph will be created, 
 *	and the number of vertices, edges and the graph's diameter will be displayed in the text area.
 *			When all the force is below the threshold value, the graph becomes stable, 
 *	and the pink "unstable" button will change to a green "stable" button.
 *			Sometimes, the graph looks stable but with a little shaking, which doesn't stop for a long time.  
 *	To fix this, increase the threshold by clicking the "+Threshold" button. 
 *	This will make the graph becomes stable easier.
 *			Sometimes, the graph becomes stable but actually not, i.e., 
 *	it becomes stable before it changes to the expected shape.  
 *	To fix this, lower the threshold by clicking the "-Threshold" button. 
 *	This will make the program takes into account the small forces that are not used before 
 *	since they are lower than the threshold.
 *			The scrollbars will rotate the graph about the x axis and y axis. 
 *			Clicking a vertex at any time will define that vertex as a base point for the distance calculation. 
 *	It will use the unweighted-edge calculation and try to calculate the shortest path from the base point to other points.  
 *	The distances are displayed on the screen along with the vertices' names.
 *			Clicking the "dijkstra" button will change the mode of distance calculation to dijkstra, 
 *	which will weighted each edge with the actual distance between vertices. 
 *	It requires selecting the base point.  After the base vertex is clicked, 
 *	the vertices names and the distances from base point will be labeled on the graph.  
 *	The shortest path to each vertices is printed on the text area.  
 *	It might be a good idea to "pause" the graph before using "dijkstra," 
 *	because the distances used are the ones at the base-point-click time, 
 *	which will continually change as the graph changes.
 *			After "pause," click "resume" button to continue the forces calculation.
 *			And easy way to see the result of a new graph is to modify the graph.txt file, save it, and then 
 *	select it again from the menu.  No need to restart the program.
 * 
 *More control-buttons
 *			"New Random" button will recreate random positions for the vertices of the current graph.
 *			"+t" and "-t" buttons will change the time interval between each update of the positions. 
 *	To make the graph change shape faster, use "-t" button.
 *			"+x", "-x", "+y", "-y" will shift the center of the graph in that direction.
 *			Note that the graph is recentered so that it is always at the middle of the screen.
 *			"+SpringF" and "-SpringF" buttons will change the spring constant. 
 *	To make the spring force stronger, which will make the graph eventually smaller, use "+SpringF"
 *			"+Threshold" and "-Threshold" will change the amount of the force that is weak enough and can be disregarded. 
 *			"Stop" button will cease the force calculation and the graph screen update.
 * 
 * @author Prapun Suksompong ps92
 * @author Supap Kirtsaeng sk158
 */
public class GraphDemo
{
	/**Method main.*/
	public static void main (String[] args)
	{		
		GraphGui gg = new GraphGui();
		//prepare user for watching the demo
		gg.ta.append("Start the demo in 2 second...");
		try{Thread.sleep(1000);}
		catch(InterruptedException ie){};
		gg.ta.append("1...");
		try{Thread.sleep(1000);}
		catch(InterruptedException ie){};
		gg.ta.append("Go!");
		gg.ta.append("\nPlease select the file name of the graph in the menu.");
		gg.gp.start();
		//System.out.println(gg.gp.g.toString());
	}
}

/**This class captures the info of a graph, and provide some functionality for it.*/
class Graph
{
	/**Vector containing all vertices*/
	Vector vertex = new Vector();
	/**Vector containing all edges. Each edge is a list of integers. */
	Vector edge = new Vector();
	/**The maximum unweighted distance */
	static int currMaxDist = 0;
	/**An array containing the coordinate of the center of mass of this graph*/
	double[] cm;

	/**Instance of class Java.util.Random use to generate pseudorandom numbers.*/
	Random random = new Random();
	/**Create a new default graph*/
	Graph()
	{
		for(int i = 0; i<4;i++)
		{
			vertex.addElement(new Vertex());
		}
		this.randomizeVertex();
		for(int i =0; i<vertex.size();i++)
		{
			edge.addElement(new Vector());
		}
		connect(0,1); connect(1,2); connect(2,0); connect(3,0); connect(3,1); connect(3,2);
		
		cm = new double[((Vertex)vertex.elementAt(0)).dimension];
	}
	/**Return a string describing this graph including vertices and edges.
	 * @return  a string describing this graph
	 */
	public String toString()
	{
		String s = "Vertex: \n";
		//print coordinate of the vertex
		for(int i=0; i<vertex.size();i++)
		{
			s += i + " : at " + ((Vertex)(vertex.elementAt(i))).toString() + " ,dist = " + ((Vertex)(vertex.elementAt(i))).dist + "\n";
		}
		//print the adjacency list
		for(int i=0; i<edge.size();i++)
		{
			s += i+": ";
			Vector v = (Vector)(edge.elementAt(i));			
			for(int j = 0; j<v.size();j++)
			{
				s += ((Integer)(v.elementAt(j))).intValue()+" ";
			}
			s += "\n";
		}
		return s;
	}
	/**Connect Vertex v1 and Vertex v2 with an undirected edge
	 * @ param v1 the first vertex
	 * @ param v2 the second vertex
	 */
	void connect(int v1, int v2)
	{
		((Vector)edge.elementAt(v1)).addElement(new Integer(v2));
		((Vector)edge.elementAt(v2)).addElement(new Integer(v1));
	}
	/**Return the vertex that can be found at the position (x,y)
	 * @param x the x coordinate
	 * @param y the y coordinate
	 * @return the vertex that can be found at the position (x,y); -1 if no vertex there
	 */
	int findVertexAt(int x, int y)
	{
		int noVertex = -1;
		double vx = 0;
		double vy = 0;
		for(int i=0; i<vertex.size();i++)
		{
			Vertex temp = (Vertex)vertex.elementAt(i);
			vx = temp.coor[0];
			vy = temp.coor[1];
			double distance = Math.sqrt(Math.pow(x-vx,2) + Math.pow(y-vy,2));
			if(distance < 5)
			{
				return i;
			}
		}
		//System.out.println(x + " not " + vx + " , " + y + " not " + vy + " , ");
		return noVertex;
	}
	/**Substract the value in the array from the current coordinate
	 * @param amount the array containing the value to be taken off
	 */
	void translateVertexBy(double amount[])
	{
		for(int i =0; i<vertex.size();i++)
		{
			Vertex p = (Vertex)(vertex.elementAt(i));
			for(int j=0; j<p.dimension;j++)
			{
				p.coor[j] -=amount[j];
			}
		}
	}
	/**Rotate this graph around the X axis by the amount specified in radian
	 * @param amount the amount to be rotated
	 */
	void rotateX(double amount)
	{
		for(int i =0; i<vertex.size();i++)
		{
			Vertex p = (Vertex)(vertex.elementAt(i));
			p.coor[1] = p.coor[1]*Math.cos(amount)-p.coor[2]*Math.sin(amount);
			p.coor[2] = p.coor[1]*Math.sin(amount)+p.coor[2]*Math.cos(amount);
		}		
	}
	/**Rotate this graph around the Y axis by the amount specified in radian
	 * @param amount the amount to be rotated
	 */
	void rotateY(double amount)
	{
		for(int i =0; i<vertex.size();i++)
		{
			Vertex p = (Vertex)(vertex.elementAt(i));
			p.coor[2] = p.coor[2]*Math.cos(amount)-p.coor[0]*Math.sin(amount);
			p.coor[0] = p.coor[2]*Math.sin(amount)+p.coor[0]*Math.cos(amount);
		}		
	}
	/**Calculate the center of mass of this graph
	 */
	void calculateCm()
	{
		for(int j=0; j<cm.length;j++)
		{
			cm[j] = 0;
			for(int i=0;i<vertex.size();i++)
			{
				cm[j] += ((Vertex)vertex.elementAt(i)).coor[j];
			}
			cm[j] = (cm[j] / vertex.size());
		}		
	}
	/**Reset all graph-function variables to their default value.  Do this before executing new operation.*/
	void reset()
	{
		currMaxDist = 0;
		for(int i=0;i<vertex.size();i++)
		{
			Vertex p = (Vertex)vertex.elementAt(i);
			p.known = false;
			p.dist = Vertex.INF;
			p.via = Vertex.noPath;
		}
	}
	double distance(int m, int n)
	{
		return (new MVector((Vertex)(vertex.elementAt(m)),(Vertex)(vertex.elementAt(n)))).magnitude();
	}
	/**Solve the single-source shortest-path problem
	 * @param base the base Vertex for this calculation
	 */
	void dijkstra(int base)
	{
		reset();
		Vertex v,w;
		((Vertex)(vertex.elementAt(base))).dist = 0;
		for(;;)
		{
			//find the smallest unknown distance vertex
			int minUnknownV = Vertex.noPath;
			int minUnknownValue = Vertex.INF;
			for(int i=0;i<vertex.size();i++)
			{
				Vertex p = (Vertex)vertex.elementAt(i);
				if(!p.known)
				{
					if(p.dist < minUnknownValue )
					{
						minUnknownValue = p.dist;
						minUnknownV = i;
					}
				}
			}
			if(minUnknownV == Vertex.noPath)
				break;
			v = (Vertex)vertex.elementAt(minUnknownV);
			v.known = true;
			Vector tempV = (Vector)edge.elementAt(minUnknownV);
			for(int i=0; i<tempV.size(); i++)
			{
				int temp = ((Integer)tempV.elementAt(i)).intValue();
				w = (Vertex)(vertex.elementAt(temp));
				if(!w.known)
				{
					//System.out.println("Considering : " + temp);
					int d = (int)distance(minUnknownV,temp);
					//System.out.println("Update distance at : " + temp);
					if(v.dist + d < w.dist)
					{
						w.dist = v.dist + d;
						if(w.dist>currMaxDist)
						{
							currMaxDist = w.dist;
						}
						w.via = minUnknownV;
					}
				}
			}				
		}
		for(int k = 0; k<vertex.size();k++)
		{
			if(k!=base)
			{
				GraphGui.ta.append("\nThe path from vertex " + base + " to vertex " + k + " is : ");
				printPath(k);
				GraphGui.ta.append(" ; Distance = " + ((Vertex)(vertex.elementAt(k))).dist);
			}
		}
	}
	/**Print path to the input vertex on the text area.
	 * @param toVertex The destination
	 */
	void printPath(int toVertex)
	{
		int temp = ((Vertex)vertex.elementAt(toVertex)).via;
		if(temp!=Vertex.noPath)
		{
			printPath(temp);
		}
		GraphGui.ta.append(" "+toVertex);
	}
	/**Assign the distance variable to each vertex with respect to the baseVertex
	 * @param baseVertex base Vertex
	 */
	void AssignDist2(int baseVertex)
	{
		Vertex v, w;
		((Vertex)(vertex.elementAt(baseVertex))).dist = 0;
		for(int currDist=0; currDist < vertex.size(); currDist++)
		{
			for(int i = 0; i<vertex.size();i++)
			{
				v = (Vertex)(vertex.elementAt(i));
				if(!v.known && v.dist==currDist)
				{
					v.known = true;
					//System.out.println("Know vertex " + i);
					Vector tempV = (Vector)edge.elementAt(i);
					for(int j =0; j<tempV.size();j++)
					{
						int temp = ((Integer)tempV.elementAt(j)).intValue();
						//System.out.println("Consider vertex" + temp + " to " + i);
						w = (Vertex)(vertex.elementAt(temp));
						if(w.dist==Vertex.INF)
						{
							w.dist = currDist+1;
							currMaxDist=currDist+1;
						}
					}
				}
			}
		}
	}
	void AssignDist(int baseVertex)
	{
		Vertex v, w;
		((Vertex)(vertex.elementAt(baseVertex))).dist = 0;
		((Vertex)(vertex.elementAt(baseVertex))).known = true;
		boolean allKnown = false;
		while(!allKnown)
		{
			allKnown = true;
			for(int i = 0; i<vertex.size();i++)
			{
				v = (Vertex)(vertex.elementAt(i));
				if(v.known)
				{
					Vector adjList = (Vector)edge.elementAt(i);
					for(int j =0; j<adjList.size();j++)
					{
						int adjV = ((Integer)adjList.elementAt(j)).intValue();
						w = (Vertex)(vertex.elementAt(adjV));
						if(!w.known)
						{
							w.dist = v.dist+1;
							w.known = true;
							currMaxDist=w.dist;
						}
					}
				}
				else{allKnown = false;}
			}
		}
	}
	/**Give each vertex a new random coordinate*/
	void randomizeVertex()
	{
		for(int i =0; i<vertex.size();i++)
		{
			Vertex p = (Vertex)(vertex.elementAt(i));
			for(int j=0; j<p.dimension;j++)
			{
				p.coor[j] = random.nextDouble()*100;
			}
		}
	}
}

/**A GUI class containing user interface.*/
class GraphGui extends Frame implements ActionListener, AdjustmentListener, ItemListener
{
	/**Color of the background.*/
	static Color backgroundColor = new Color(245,197,141);
	/**Name of all Buttons*/
	String[] bName = {"Resume", "Stop", "New Random", "Pause" , "+t", "-t", "+x", "-x", "+y", "-y" ,"+SpringF", "-SpringF",  "+Threshold","-Threshold", "Dijkstra"};
	/**Name of possible graph file*/
	String[] gName = {"graph","Cube", "BinaryTree","TrinaryTree","dodecahedron","icosahedron","octahedron", "hypercube"};
	/**TextArea for displaying messages*/
	static TextArea ta = new TextArea(3,3);
	/**Refion for displaying the graph*/
	GraphPanel gp;
	/**Number of all controlling buttons*/
	int nb = bName.length;
	/**Panel the keep all buttons and the menu*/
	Panel bp = new Panel();
	/**Control panel for controlling the graph, containing button and scrollbar*/
	Panel control = new Panel();
	/**Buttons used for controlling the graph*/
	Button[] bArray = new Button[nb];
	/**Scrollbar in the X direction*/
	Scrollbar sX;
	/**Scrollbar in the Y direction*/
	Scrollbar sY;
	/**Choice for choosing the file name of the graph*/
	Choice cGraph;
	/**Starting value for scrollbar in the X direction*/
	int xScrollValue = 0;
	/**Starting value for scrollbar in the Y direction*/	
	int yScrollValue = 0;
	/**Default Rotation amount*/
	static double amountRotate = 0.05;
	/**Width*/
	static int w = 700;
	/**Height*/
	static int h = 500;
	/**Create a frame containing the graph and all the controls*/
	GraphGui()
	{
		super("Graph");
		setSize(w,h);
		addWindowListener(
			new WindowAdapter()
			{
				public void windowClosing(WindowEvent e){GraphPanel.runable = false; dispose(); System.exit(0);}
			}
		);
		setBackground(backgroundColor);
		gp = new GraphPanel();
		setLayout( new BorderLayout() );
		control.setSize(500,50);
		control.setLayout(new BorderLayout());
		bp.setSize(500,50);
		GridLayout G = new GridLayout();
		G.setRows(2);
		bp.setLayout(G);		
		for(int i=0; i<nb;i++)
		{
			bArray[i] = new Button(bName[i]);
			bArray[i].addActionListener(this);
			bp.add(bArray[i]);
		}
		sY = new Scrollbar(Scrollbar.HORIZONTAL, 0, 20, -90, 90);
		sY.addAdjustmentListener(this);
		sX = new Scrollbar(Scrollbar.VERTICAL, 0, 20, -90, 90);
		sX.addAdjustmentListener(this);
		cGraph = new Choice();
		cGraph.addItemListener(this);
		for(int i=0;i<gName.length;i++)
		{
			cGraph.add(gName[i]);
		}
		bp.add(gp.idButton);
		bp.add(cGraph);
		control.add(sY,BorderLayout.NORTH);
		control.add(bp,BorderLayout.CENTER);
		
		add(ta, BorderLayout.NORTH);
		add(gp, BorderLayout.CENTER);
		add(control, BorderLayout.SOUTH);
		add(sX, BorderLayout.EAST);
		setVisible(true);
	}
	/**Response to the input from ActionListener.
	 * @param e the input ActionEvent
	 */
	public void actionPerformed(ActionEvent e)
	{
		if(e.getActionCommand().equals("Resume"))
		{
			GraphPanel.stable = false;
		}
		else if(e.getActionCommand().equals("Stop"))
		{
			GraphPanel.runable = false;
		}
		else if(e.getActionCommand().equals("Pause"))
		{
			GraphPanel.stable = true;
		}
		else	 if(e.getActionCommand().equals("New Random"))
		{
			gp.g.randomizeVertex();
			GraphPanel.stable = false;
		}
		else	 if(e.getActionCommand().equals("+SpringF"))
		{
			MVector.SFC = (MVector.SFC)*3.0/2.0;
			GraphPanel.stable = false;
		}
		else	 if(e.getActionCommand().equals("-SpringF"))
		{
			MVector.SFC = (MVector.SFC)*2.0/3.0;
			GraphPanel.stable = false;
		}
		else	 if(e.getActionCommand().equals("+Threshold"))
		{
			GraphPanel.threshold = GraphPanel.threshold*10.0;
			GraphPanel.stable = false;
		}		
		else	 if(e.getActionCommand().equals("-Threshold"))
		{
			GraphPanel.threshold = GraphPanel.threshold*0.1;
			GraphPanel.stable = false;
		}
		else	 if(e.getActionCommand().equals("Dijkstra"))
		{
			ta.append("\nPlease select a base Vertex.");
			GraphPanel.dijkstra = true;
		}
		else	 if(e.getActionCommand().equals("+t"))
		{
			gp.t = 3*gp.t/2;
		}
		else	 if(e.getActionCommand().equals("-t"))
		{
			gp.t = gp.t*2/3;
		}
		else	 if(e.getActionCommand().equals("+x"))
		{
			gp.oX += 20;
			gp.repaint();
		}		
		else	 if(e.getActionCommand().equals("-x"))
		{
			gp.oX -= 20;
			gp.repaint();
		}		
		else	 if(e.getActionCommand().equals("+y"))
		{
			gp.oY -= 20;
			gp.repaint();
		}		
		else	 if(e.getActionCommand().equals("-y"))
		{
			gp.oY += 20;
			gp.repaint();
		}		
	}
	/**Response to the input from AdjustmentListener.
	 * @param e the input AdjustmentEvent
	 */
	public void adjustmentValueChanged(AdjustmentEvent e)
	{
		if(e.getSource()==sX)
		{
			int temp = e.getValue();
			gp.g.rotateX((temp - xScrollValue)*2*Math.PI/360);
			xScrollValue = temp;
			gp.repaint();			
			GraphPanel.stable = false;
		}
		else	if(e.getSource()==sY)
		{
			int temp = e.getValue();
			gp.g.rotateY((temp - yScrollValue)*2*Math.PI/360);
			yScrollValue = temp;
			gp.repaint();
			GraphPanel.stable = false;
		}
	}
	/**Response to the input from ItemListener.
	 * @param e the input ItemEvent
	 */	
	public void itemStateChanged(ItemEvent e)
	{
		for(int i=0;i<gName.length;i++)
		{
			if(((String)(e.getItem())).equals(gName[i]))
			{
				try
				{
					ReadFile rf = new ReadFile(gName[i]+".txt");
					gp.g = rf.getGraph();
					gp.vertex = gp.g.vertex;
					gp.edge = gp.g.edge;
					GraphPanel.stable = false;
				}
				catch(FileNotFoundException fe)
				{
					ta.append("\nFile Not Found!");
				}
				catch(IOException ie)
				{
				}
				break;
			}
		}
	}
}

/**This class create a region that displays the graph*/
class GraphPanel extends Panel
{
	/**List of all vertices of the graph*/
	Vector vertex;
	/**Adjacency List for the graph*/
	Vector edge;
	/**The graph to be displayed*/
	Graph g;
	/**The lowest magnitude of the force that is considered unstable*/
	static double threshold = 1;
	/**The unit of transition of the vertex when the it is unbalanced*/
	static double move = 1;
	/**True when all verteices are stable*/
	static boolean stable = false;
	/**False when the force interaction are no longer need*/
	static boolean runable = true;
	/**True when moused just clicked*/
	boolean mouseClicked = false;
	/**True when performing dijkstra*/
	static boolean dijkstra = false;
	/**The x positon of the mouse, updated when mouse-clicked*/
	int mouseX =0;
	/**The x positon of the mouse, updated when mouse-clicked*/
	int mouseY = 0;
	/**The width of the drawing portion of this project*/
	static int w = GraphGui.w;
	/**The width of the drawing portion of this project*/
	static int h = 500;	
	/**The x component of the position where the center of mass of the graph should be*/
	static int oX = GraphGui.w/2;
	/**The y component of the position where the center of mass of the graph should be*/
	static int oY = GraphGui.h/3;
	/**Array to keep the center of mass of this graph*/
	double[] cm; //center of mass
	/**Indication that the baseVertex has not been specified*/
	int baseVertex = -1;
	/**Color of the displaying region of the graph*/
	static Color backgroundColor = new Color(176,244,229);
	/**Color of a Vertex telling no further information*/
	static Color VertexColor = new Color(253,53,241);
	/**Color of a base Vertex*/
	static Color baseVertexColor = new Color(77,100,151);
	/**Color of the edge */
	static Color lineColor = Color.red;
	/**Radius of a Vertex*/
	static int VertexRadius = 10;
	/**Button to indicate the the graph is stable*/
	static Button idButton = new Button();
	/**Color of a stable graph button*/
	static Color cStable = new Color(148,246,128);
	/**Color of an unstable graph button*/
	static Color cUnstable = new Color(247,115,199);
	/**Time interval between each update*/
	static int t = 100;
	/**Ceate a region that displays the graph*/
	GraphPanel()
	{
		//get the default graph to display first
		g = new Graph();
		vertex = g.vertex;
		edge = g.edge;
		setSize(w,h);
		cm = new double[((Vertex)vertex.elementAt(0)).dimension];
		setBackground(backgroundColor);
		addMouseListener(
			new MouseAdapter()
			{
				public void mouseClicked(MouseEvent e)
				{
					//System.out.println("Mouse clicked!");
					mouseClicked = true;
					mouseX = e.getX();
					mouseY = e.getY();
					repaint();
				}
			}
		);
	}
	/**Start generating the current graph. Pause when all edges are stable.*/
	void start()
	{
		runable = true;
		MVector temp;
		boolean allLow = false;
		boolean low = true;
		while(runable)
		{
			if(!stable)
			{
				idButton.setBackground(cUnstable);
				idButton.setLabel("Unstable");
				low = true;
				allLow = true;
				for(int j=0; j<vertex.size();j++)
				{
					temp = MVector.calculateTotalForce(j, edge, vertex);
					double magnitude = temp.magnitude();
					low = magnitude < threshold;
					allLow = allLow && low;
					if(!low)
					{
						for(int k =0; k<temp.dimension;k++)
						{
							if(temp.comp[k]>0)
							{
								((Vertex)vertex.elementAt(j)).translate(k,move);
							}
							else if(temp.comp[k]<0)
							{
								((Vertex)vertex.elementAt(j)).translate(k,-move);
							}
						}
					}
					if(allLow)
					{
						stable = true;
						idButton.setBackground(cStable);
						idButton.setLabel("Stable");
					}
				}
				try
				{
					Thread.sleep(t);
					repaint();
				}
				catch(InterruptedException e){}
			}
		}
	}
	/**Paint all edges and all vertices on GUI window.
	 * @param graphics graphics to be drawn
	 */
	public void paint(Graphics graphics)
	{
		//only executed when mouse clicked
		if(mouseClicked)
		{
			int baseVertexTemp = g.findVertexAt((int)(mouseX-oX),(int)(mouseY-oY)); //**
			//System.out.println(baseVertexTemp);
			if(baseVertexTemp!=-1)
			{
				if(baseVertex!=-1)
				{
					((Vertex)(vertex.elementAt(baseVertex))).dist = Vertex.INF;
				}
				baseVertex=baseVertexTemp;
				g.reset();
				if(dijkstra)
				{
					g.dijkstra(baseVertex);
					dijkstra = false;
				}
				else
				{
					g.AssignDist(baseVertex);
				}
			}
			
		}
		g.calculateCm();
		g.translateVertexBy(g.cm);//**
		drawAllEdge(graphics);
		if(this.mouseClicked)
		{
			this.drawAllVertexW(graphics);
			mouseClicked = false;
		}
		drawAllVertex(graphics);
		
		cm=AM.zero(cm.length);
	}
	/**Draw vertex
	 * @param p Vertex to be drawn
	 * @param g Graphics to be drawn on
	 * @param index the index of Vertex p
	 */
	void drawVertex(Vertex p, Graphics g, int index)
	{
		g.setColor(VertexColor);
		if(p.dist==0)
		{
			g.setColor(baseVertexColor);
		}
		if(p.dist!=Vertex.INF)
		{
			g.setColor(Color.blue);
			g.drawString("Vertex: " +index + " ; Dist: " +p.dist,(int)(p.coor[0]-VertexRadius+oX),(int)(p.coor[1]-VertexRadius+oY)); //**
			g.setColor(new Color(77,100+(int)((p.dist*155)/Graph.currMaxDist),151));
		}
		
		g.fillOval( (int)(p.coor[0]-VertexRadius+oX),(int)(p.coor[1]-VertexRadius+oY),  2*VertexRadius,2*VertexRadius); //**

	}
	void drawAllVertexW(Graphics graphics)
	{
		for(int i=0;i<g.currMaxDist;i++)
		{
			boolean find=false;
			for(int j=0; j < vertex.size();j++)
			{
				if(((Vertex)vertex.elementAt(j)).dist==i)
				{
					drawVertex((Vertex)vertex.elementAt(j), graphics, j);
					find = true;
				}
			}
			if(find)
				try{Thread.sleep(300);} catch(InterruptedException ie){}
		}
	}
	/**Draw all vertices
	 * @param graphics the graphics to be drawn on
	 */
	void drawAllVertex(Graphics graphics)
	{
		for(int i=0; i < vertex.size();i++)
		{
			drawVertex((Vertex)vertex.elementAt(i), graphics, i);
		}
	}
	/**Draw an edge between two vertices
	 * @param p1 the Vertex on one end
	 * @param p2 the Vertex on the other end
	 * @param g the graphics to be drawn on
	 */
	void drawEdge(Vertex p1, Vertex p2, Graphics g)
	{
		g.setColor(lineColor); 
		g.drawLine((int)(p1.coor[0]+oX),(int)(p1.coor[1]+oY),(int)(p2.coor[0]+oX),(int)(p2.coor[1]+oY)); //**
	}
	/**Draw all edge
	 * @param graphics the graphics to be drawn on
	 */		
	void drawAllEdge(Graphics graphics)	
	{
		for(int i = 0; i < edge.size(); i++)
		{
			Vector temp = (Vector)edge.elementAt(i);
			int c = temp.size();
			for(int j=0;j<c;j++)
			{
				int k = ((Integer)temp.elementAt(j)).intValue();
				drawEdge((Vertex)vertex.elementAt(i),(Vertex)vertex.elementAt(k),graphics);
			}
		}
	}
}

/**An instance of this class contains the information of a vertex on the graph.*/
class Vertex
{
	/**Infinite distance */
	static int INF = 1000000;
	/**Default dimension of vertex*/
	static int defaultDimension = 3;
	/**Default value for unknown vertex */
	static int noPath = -2;
	/**The dimension of vertex */
	int dimension = defaultDimension;
	/**The coordinate of the vertex	 */
	double[] coor;
	/**Distance of vertex	 */
	int dist = INF;
	/**Path of vertex	 */
	int via = noPath;
	/**Flags true if the Vertex is known	 */
	boolean known = false;
	
	/**Construct an n-dimension vertex.
	 * @param dimension the dimension of the vertex
	 */
	Vertex(int dimension)
	{
		this.dimension = dimension;
		coor = new double[dimension];
	}
	/**Construct a vertex of default dimension.*/
	Vertex()
	{
		this(defaultDimension);
	}
	/**Construct a vertex of dimension equals to the length of input array with corresponding coordinate.
	 * @param a the input array of corresponding coordinate
	 */
	Vertex(double[] a)
	{
		dimension = a.length;
		coor = AM.copyOf(a);
	}
	/**Construct a copy of input vertex.
	 * @param p the input vertex
	 */
	Vertex(Vertex p)
	{
		dimension = p.dimension;
		coor = AM.copyOf(p.coor);
	}
	/**Set coordinate of the vertex to the location input by an array
	 * @param a the input array
	 */
	void setCoor(double[] a)
	{
		if(a.length!=dimension)
		{
			//dosomething
			return;
		}
		coor = AM.copyOf(a);
	}
	/**Give the information of vertex to a String
	 * @return the coordinate of the vertex
	 */
	public String toString()
	{
		String s = "(";
		for(int i =0; i < dimension-1; i++)
		{
			s += coor[i]+",";
		}
		s += coor[dimension-1]+")";
		return s;
	}
	/**Check if two instances of Vertex are equal.
	 * @return true if both Vertex has the same coordinate, false otherwise.
	 */
	boolean equals(Vertex p)
	{
		return AM.equals(coor, p.coor);
	}
	/**Set the coordinate of vertex on given axis to the given value.
	 * @param axis the axis
	 * @param value the value on that axis
	 */
	void setCoor(int axis, double value)
	{
		coor[axis] = value;
	}
	/**Translate the vertex along the given axis in positive direction by given amount
	 * @param axis the axis of translation
	 * @param amoung the distance of translation
	 */
	void translate(int axis, double amount)
	{
		coor[axis] += amount;
	}
	/**Reset the vertex to be an unknown vertex at infinite distance.
	 */
	void reset()
	{
		dist = INF;
		known = false;
	}
}

/**Array manipulator */
class AM
{
	//---------------------------------------------------------------------------------------------------------------------------------
	/**Return a new copy of int[] a
	 * @param a input array
	 * @return an array of integer
	*/
	static int[] copyOf(int[] a)
	{
		int[] copy = new int[a.length];
		for(int i = 0; i<a.length; i++)
		{
			copy[i] = a[i];
		}
		return copy;
	}
	/**Return a new copy of double[] a
	 * @param a input array
	 * @return an array of double
	 */
	static double[] copyOf(double[] a)
	{
		double[] copy = new double[a.length];
		for(int i = 0; i<a.length; i++)
		{
			copy[i] = a[i];
		}
		return copy;
	}
	//---------------------------------------------------------------------------------------------------------------------------------
	//---------------------------------------------------------------------------------------------------------------------------------
	/**Print to console window an array of integer
	 * @param a the array to be printed*/
	static void print(int[] a)
	{
		for(int i = 0; i<a.length; i++)
		{
			System.out.print(a[i] + " ");
		}
		System.out.print("\n");
	}
	/**Print to console window an array of double
	 * @param a the array to be printed
	 */
	static void print(double[] a)
	{
		for(int i = 0; i<a.length; i++)
		{
			System.out.print(a[i] + " ");
		}
		System.out.print("\n");
	}
	//---------------------------------------------------------------------------------------------------------------------------------
	/**Convert int[] a to string
	 * @param a input array
	 * @return String representation of the array*/
	static String toString(int[] a)
	{
		String temp = new String("");
		for(int i = 0; i<a.length; i++)
		{
			temp += a[i];
		}
		return temp;
	}
	/**Convert double[] a to String
	 * @param a input array
	 * @return String representation of the array
	 */
	static String toString(double[] a)
	{
		String temp = new String("");
		for(int i = 0; i<a.length; i++)
		{
			temp += a[i];
		}
		return temp;
	}
	/**Method to check if two array of integers are equals
	 * @param a an array to be compared
	 * @param b the other array to be compared
	 * @return true if all elements are equal, false otherwise.
	 */
	static boolean equals(int[] a, int[] b)
	{
		if(a.length!=b.length) return false;
		for(int i=0; i<a.length; i++)
		{
			if(a[i]!=b[i]) 
			{
				return false;
			}
		}
		return true;
	}
	/**Method to check if two array of double are equals
	 * @param a an array to be compared
	 * @param b the other array to be compared
	 * @return true if all elements are equal, false otherwise.
	 */
	static boolean equals(double[] a, double[] b)
	{
		if(a.length!=b.length) return false;
		for(int i=0; i<a.length; i++)
		{
			if((int)a[i]!=(int)b[i]) 
			{
				return false;
			}
		}
		return true;
	}
	/**Create an array of size n with all element equals zero.
	 * @param n dimension of the output array
	 * @return a zero array of size n
	 */
	static double[] zero(int n)
	{
		double[] temp = new double[n];
		for(int i=0; i<n;i++)
		{
			temp[i] = 0;
		}
		return temp;
	}
}


/**Math Vector. This class allows you to manipulate its instances with mathematical vector operations.
 */
class MVector
{
	/**charge force constant	 */
	static double CFC = 2500000;
	/**spring force constant	 */
	static double SFC = 1;
	/**spring natural length	 */
	static double naturalL = 100;
	/**default dimension	 */
	static int defaultDimension = 3;
	/**dimension of the vector	 */
	static int dimension = defaultDimension;
	/**the array contain coordinate of vector	 */
	double[] comp;
	/**Create a new (zero) vector with the specified dimension
	 *@param dimension dimension of this vector 
	 */
	MVector(int dimension)
	{
		this.dimension = dimension;
		comp = new double[dimension];
	}
	/**Create a new (zero) vector with the default dimension*/
	MVector()
	{
		this(defaultDimension);
	}
	/**Create a new vector that has components as specified in the array
	 * @param a double[] array storing the value for each component of the vector
	 */
	MVector(double[] a)
	{
		dimension = a.length;
		comp = AM.copyOf(a);
	}
	/**Create a new vector from Vertex p1 to Vertex p2
	 * @param p1 the starting Vertex
	 * @param p2 the ending Vertex
	 */
	MVector(Vertex p1, Vertex p2)
	{
		dimension = p1.dimension;
		comp = new double[dimension];
		for(int i=0; i<dimension; i++)
		{
			comp[i] = p2.coor[i]-p1.coor[i];
		}
	}
	/**Create a new vector from origin to Vertex p
	 * @param p the ending Vertex
	 */
	MVector(Vertex p)
	{
		this(new Vertex(AM.zero(p.dimension)),p);
	}
	/**Set the value of this vector
	 * @param a double[] array containg the components' value for this vector
	 */
	void setComp(double[] a)
	{
		if(a.length!=dimension)
		{
			GraphGui.ta.append("Can't set value! Invalid dimension");
			return;
		}
		comp = AM.copyOf(a);
	}
	/**Return a string that represents this vector
	 * @return a string that represents this vector
	 */
	public String toString()
	{
		String s = "(";
		for(int i =0; i < dimension-1; i++)
		{
			s += comp[i]+",";
		}
		s += comp[dimension-1]+")";
		return s;
	}
	/**Check if two vectors are equal.
	 * @param v the other vector to be compared with
	 * @return true if both vectors are equal, false otherwise
	 */
	boolean equals(MVector v)
	{
		return AM.equals(comp, v.comp);
	}
	/**Set coordinate of vector on given axis to given value
	 * @param axis the given axis
	 * @param value the disire value
	 */
	void setComp(int axis, double value)
	{
		comp[axis] = value;
	}
	/**Calculate the magnitude of vector.
	 * @return magnitude of the vector
	 */
	double magnitude()
	{
		double temp =0;
		for(int i=0;i<dimension;i++)
		{
			temp += Math.pow(comp[i],2);
		}
		return Math.sqrt(temp);
	}
	/**Add the current vector to the given vector.
	 * @param v the given vector
	 * @return the sum of this vector and the given vector
	 */
	MVector add(MVector v)
	{
		MVector temp = new MVector(dimension);
		for(int i =0; i<dimension;i++)
		{
			temp.comp[i] = this.comp[i]+v.comp[i];
		}
		return temp;
	}
	/**Scale the vector by multipling it by a given value
	 * @param k the value to be multiplied to the vector
	 * @return the product vector
	 */
	MVector scale(double k)
	{
		MVector temps = new MVector(dimension);
		for(int i =0; i<dimension;i++)
		{
			temps.comp[i] = this.comp[i]*k;
		}
		return temps;		
	}
	/**Calculate charge force between two vertices.
	 * @param vertex the Vector containing all vertices
	 * @param index the position of one vertex in Vector
	 * @param from the position of the other vertex in Vector
	 * @return the force vector
	 */
	static MVector calculateChargeForce(Vector vertex, int index, int from)
	{
		MVector force = new MVector((Vertex)vertex.elementAt(from) , (Vertex)vertex.elementAt(index));
		double m = force.magnitude();
		force = force.scale(  CFC/(Math.pow(m,3)) );
		return force;		
	}
	/**Calculate spring force between two vertices.
	 * @param vertex the Vector containing all vertices
	 * @param index the position of one vertex in Vector
	 * @param from the position of the other vertex in Vector
	 * @return the force vector
	 */
	static MVector calculateSpringForce(Vector vertex, int index, int from)
	{
		MVector force = new MVector((Vertex)vertex.elementAt(index) , (Vertex)vertex.elementAt(from));
		double m = force.magnitude();
		force = force.scale( SFC*(m-naturalL)/m );
		return force;
	}
	/**Calculate charge force between given vertex and all other vertices in the same vector.
	 * @param vertex the Vector containing all vertices
	 * @param index the position of given vertex in Vector
	 * @return the force vector
	 */
	static MVector calculateChargeForce(int index, Vector vertex)
	{
		MVector tempf = new MVector(dimension);
		for(int i=0;i<vertex.size(); i++)
		{
			if(i!=index)
			{
				MVector force = calculateChargeForce(vertex, index, i);
				tempf = tempf.add(force);
			}
		}
		return tempf;
	}	
	/**Calculate spring force between given vertex and all connected vertices on the same vector.
	 * @param vertex the Vector containing all vertices
	 * @param edge the Vector containing all edges
	 * @param index the position of given vertex in the Vector
	 * @return the force vector
	 */	
	static MVector calculateSpringForce(int index, Vector edge, Vector vertex)
	{
		
		MVector tempf = new MVector(dimension);
		Vector list = (Vector)(edge.elementAt(index));
		for(int i=0;i<list.size(); i++)
		{
				int c = ((Integer)(list.elementAt(i))).intValue(); //connected vertex
				MVector force = calculateSpringForce(vertex, index,  c);
				//System.out.println("Spring force for vertex " + index + " and " + c + "  = " + force.magnitude());
				tempf = tempf.add(force);
		}
		return tempf;
	}
	/**Calculate total force between given vertex and all other vertices on the same vector.
	 * @param vertex the Vector containing all vertices
	 * @param edge the Vector containing all edges
	 * @param index the position of given vertex in the Vector
	 * @return the force vector
	 */	
	static MVector calculateTotalForce(int index, Vector edge, Vector vertex)
	{
		MVector temp = calculateSpringForce(index,edge,vertex);
		temp = temp.add(calculateChargeForce(index,vertex));
		return temp;
	}
}
/**An instance of this class can access an input graph file. */
class ReadFile
{
	/**Input file */
	File f;
	/**FileReader */
	FileReader fr;
	/**BufferedReader */
	BufferedReader br; 
	/**Output graph */
	Graph graph;
	/**Read graph data from a graph file and make a graph
	 * @param fileName name of the graph file
	 */
	ReadFile(String fileName) throws FileNotFoundException, IOException
	{
		f = new File(fileName);
		fr = new FileReader(f);
		br = new BufferedReader(fr);
		graph = new Graph();
		makeGraph();
		//System.out.println(graph.toString());
	}
	/**Make a graph from graph data.	 */
	public  void makeGraph() throws IOException
	{
		graph.vertex = new Vector();
		graph.edge = new Vector();
		int numberOfEdge = 0;
		int numberOfVertex = 0;
		String inputLine = br.readLine();
		while(inputLine!=null )
		{
			if(!inputLine.equals("") && inputLine.charAt(0)!='%')
			{
				StringTokenizer st = new StringTokenizer(inputLine,"[], ");
				if(st.countTokens()==1)
				{
					int temp = Integer.parseInt(st.nextToken());
					if(temp == numberOfVertex)
					{
						numberOfVertex++;
						graph.vertex.addElement(new Vertex());
					}
				}
				if(st.countTokens()==2)
				{
					if(graph.edge.size()==0)
					{
						for(int i =0; i<graph.vertex.size();i++)
						{
							graph.edge.addElement(new Vector());
						}
					}
					int temp1 = Integer.parseInt(st.nextToken());
					int temp2 = Integer.parseInt(st.nextToken());
					if(temp1>=0 && temp2 >=0 && temp1 <graph.vertex.size() && temp2 <graph.vertex.size())
					{
						numberOfEdge++;
						graph.connect(temp1,temp2);
					}
				}
			}
			inputLine = br.readLine();
		}
		br.close();
		graph.randomizeVertex();
		graph.cm = new double[((Vertex)graph.vertex.elementAt(0)).dimension];
		GraphGui.ta.append("\nThis graph has " + graph.vertex.size() + " vertices and " + numberOfEdge + " edges.");
		int diameter = 0;
		for(int i=0;i<graph.vertex.size();i++)
		{
			graph.reset();
			graph.AssignDist(i);
			//System.out.println(graph.currMaxDist);
			if(graph.currMaxDist > diameter)
			{
				diameter = graph.currMaxDist;
			}
		}
		GraphGui.ta.append(" Diameter (unweighted) = " + diameter + ".");
		graph.reset();
	}
	/**Return an instance of graph
	 * @return a graph created from graph file
	 */
	public Graph getGraph()
	{
		return graph;
	}
}